@@ -6,6 +6,8 @@ const Lang = imports.lang;
 const Shell = imports.gi.Shell;
 const Mainloop = imports.mainloop;
 const St = imports.gi.St;
+const Gettext = imports.gettext.domain('gnome-shell');
+const _ = Gettext.gettext;
 
 const Main = imports.ui.main;
 const MessageTray = imports.ui.messageTray;
@@ -173,10 +175,6 @@ NotificationDaemon.prototype = {
         let source = new Source(title, pid);
         this._sources[pid] = source;
 
-        source.connect('clicked', Lang.bind(this,
-            function() {
-                source.destroy();
-            }));
         source.connect('destroy', Lang.bind(this,
             function() {
                 delete this._sources[pid];
@@ -281,7 +279,7 @@ NotificationDaemon.prototype = {
         if (notification == null) {
             notification = new MessageTray.Notification(source, summary, body, { icon: iconActor });
             ndata.notification = notification;
-            notification.connect('dismissed', Lang.bind(this,
+            notification.connect('clicked', Lang.bind(this,
                 function(n) {
                     this._emitNotificationClosed(id, NotificationClosedReason.DISMISSED);
                 }));
@@ -296,6 +294,7 @@ NotificationDaemon.prototype = {
         }
 
         if (actions.length) {
+            notification.setUseActionIcons(hints['action-icons'] == true);
             for (let i = 0; i < actions.length - 1; i += 2)
                 notification.addButton(actions[i], actions[i + 1]);
         }
@@ -319,14 +318,15 @@ NotificationDaemon.prototype = {
     GetCapabilities: function() {
         return [
             'actions',
+            'action-icons',
             'body',
             // 'body-hyperlinks',
             // 'body-images',
             'body-markup',
             // 'icon-multi',
             'icon-static',
+            'persistence',
             // 'sound',
-            'x-gnome-icon-buttons'
         ];
     },
 
@@ -347,14 +347,14 @@ NotificationDaemon.prototype = {
         for (let id in this._sources) {
             let source = this._sources[id];
             if (source.app == tracker.focus_app) {
-                source.destroy();
+                source.activated();
                 return;
             }
         }
     },
 
     _actionInvoked: function(notification, action, source, id) {
-        source.destroy();
+        source.activated();
         this._emitActionInvoked(id, action);
     },
 
@@ -375,7 +375,7 @@ NotificationDaemon.prototype = {
     _onTrayIconAdded: function(o, icon) {
         let source = this._sources[icon.pid];
         if (!source)
-            source = this._newSource(icon.title, icon.pid);
+            source = this._newSource(icon.title || icon.wm_class || _("Unknown"), icon.pid);
         source.setTrayIcon(icon);
     },
 
@@ -404,6 +404,7 @@ Source.prototype = {
             this.title = this.app.get_name();
         else
             this.useNotificationIcon = true;
+        this._isTrayIcon = false;
     },
 
     notify: function(notification, icon) {
@@ -430,11 +431,18 @@ Source.prototype = {
     setTrayIcon: function(icon) {
         this._setSummaryIcon(icon);
         this.useNotificationIcon = false;
+        this._isTrayIcon = true;
     },
 
-    clicked: function() {
+    _notificationClicked: function(notification) {
+        notification.destroy();
         this.openApp();
-        MessageTray.Source.prototype.clicked.call(this);
+        this.activated();
+    },
+
+    activated: function() {
+        if (!this._isTrayIcon)
+            this.destroy();
     },
 
     openApp: function() {
