@@ -54,8 +54,7 @@ enum
 
   PROP_LABEL,
   PROP_TOGGLE_MODE,
-  PROP_CHECKED,
-  PROP_TRANSITION_DURATION
+  PROP_CHECKED
 };
 
 enum
@@ -72,16 +71,10 @@ struct _StButtonPrivate
 {
   gchar            *text;
 
-  guint8            old_opacity;
-
   guint             is_pressed : 1;
-  guint             is_hover : 1;
   guint             is_checked : 1;
   guint             is_toggle : 1;
-
-  gint              transition_duration;
-
-  ClutterAnimation *animation;
+  guint             has_grab : 1;
 
   gint              spacing;
 };
@@ -95,13 +88,13 @@ st_button_update_label_style (StButton *button)
 {
   ClutterActor *label;
 
-  label = st_bin_get_child ((StBin*) button);
+  label = st_bin_get_child (ST_BIN (button));
 
   /* check the child is really a label */
   if (!CLUTTER_IS_TEXT (label))
     return;
 
-  _st_set_text_from_style ((ClutterText*) label, st_widget_get_theme_node (ST_WIDGET (button)));
+  _st_set_text_from_style (CLUTTER_TEXT (label), st_widget_get_theme_node (ST_WIDGET (button)));
 }
 
 static void
@@ -116,7 +109,7 @@ st_button_style_changed (StWidget *widget)
   ST_WIDGET_CLASS (st_button_parent_class)->style_changed (widget);
 
   spacing = 6;
-  st_theme_node_get_length (theme_node, "border-spacing", FALSE, &spacing);
+  st_theme_node_lookup_length (theme_node, "border-spacing", FALSE, &spacing);
   priv->spacing = (int)(0.5 + spacing);
 
   /* update the label styling */
@@ -130,15 +123,32 @@ st_button_style_changed (StWidget *widget)
 }
 
 static void
-st_button_real_pressed (StButton *button)
+st_button_press (StButton *button)
 {
-  st_widget_add_style_pseudo_class ((StWidget*) button, "active");
+  if (button->priv->is_pressed)
+    return;
+
+  button->priv->is_pressed = TRUE;
+  st_widget_add_style_pseudo_class (ST_WIDGET (button), "active");
 }
 
 static void
-st_button_real_released (StButton *button)
+st_button_release (StButton *button,
+                   gboolean  clicked)
 {
-  st_widget_remove_style_pseudo_class ((StWidget*) button, "active");
+  if (!button->priv->is_pressed)
+    return;
+
+  button->priv->is_pressed = FALSE;
+  st_widget_remove_style_pseudo_class (ST_WIDGET (button), "active");
+
+  if (clicked)
+    {
+      if (button->priv->is_toggle)
+        st_button_set_checked (button, !button->priv->is_checked);
+
+      g_signal_emit (button, button_signals[CLICKED], 0);
+    }
 }
 
 static gboolean
@@ -150,14 +160,10 @@ st_button_button_press (ClutterActor       *actor,
   if (event->button == 1)
     {
       StButton *button = ST_BUTTON (actor);
-      StButtonClass *klass = ST_BUTTON_GET_CLASS (button);
-
-      button->priv->is_pressed = TRUE;
 
       clutter_grab_pointer (actor);
-
-      if (klass->pressed)
-        klass->pressed (button);
+      button->priv->has_grab = TRUE;
+      st_button_press (button);
 
       return TRUE;
     }
@@ -172,25 +178,47 @@ st_button_button_release (ClutterActor       *actor,
   if (event->button == 1)
     {
       StButton *button = ST_BUTTON (actor);
-      StButtonClass *klass = ST_BUTTON_GET_CLASS (button);
+      gboolean is_click;
 
-      if (!button->priv->is_pressed)
-        return FALSE;
+      is_click = button->priv->has_grab && st_widget_get_hover (ST_WIDGET (button));
+      st_button_release (button, is_click);
 
-      clutter_ungrab_pointer ();
-
-      if (button->priv->is_toggle)
+      if (button->priv->has_grab)
         {
-          st_button_set_checked (button, !button->priv->is_checked);
+          button->priv->has_grab = FALSE;
+          clutter_ungrab_pointer ();
         }
 
-      button->priv->is_pressed = FALSE;
+      return TRUE;
+    }
 
-      if (klass->released)
-        klass->released (button);
+  return FALSE;
+}
 
-      g_signal_emit (button, button_signals[CLICKED], 0);
+static gboolean
+st_button_key_press (ClutterActor    *actor,
+                     ClutterKeyEvent *event)
+{
+  st_widget_hide_tooltip (ST_WIDGET (actor));
 
+  if (event->keyval == CLUTTER_KEY_space ||
+      event->keyval == CLUTTER_KEY_Return)
+    {
+      st_button_press (ST_BUTTON (actor));
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gboolean
+st_button_key_release (ClutterActor    *actor,
+                       ClutterKeyEvent *event)
+{
+  if (event->keyval == CLUTTER_KEY_space ||
+      event->keyval == CLUTTER_KEY_Return)
+    {
+      st_button_release (ST_BUTTON (actor), TRUE);
       return TRUE;
     }
 
@@ -202,12 +230,19 @@ st_button_enter (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
 {
   StButton *button = ST_BUTTON (actor);
+  gboolean ret;
 
-  st_widget_add_style_pseudo_class ((StWidget*) button, "hover");
+  ret = CLUTTER_ACTOR_CLASS (st_button_parent_class)->enter_event (actor, event);
 
-  button->priv->is_hover = 1;
+  if (button->priv->has_grab)
+    {
+      if (st_widget_get_hover (ST_WIDGET (button)))
+        st_button_press (button);
+      else
+        st_button_release (button, FALSE);
+    }
 
-  return CLUTTER_ACTOR_CLASS (st_button_parent_class)->enter_event (actor, event);
+  return ret;
 }
 
 static gboolean
@@ -215,24 +250,19 @@ st_button_leave (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
 {
   StButton *button = ST_BUTTON (actor);
+  gboolean ret;
 
-  button->priv->is_hover = 0;
+  ret = CLUTTER_ACTOR_CLASS (st_button_parent_class)->leave_event (actor, event);
 
-  if (button->priv->is_pressed)
+  if (button->priv->has_grab)
     {
-      StButtonClass *klass = ST_BUTTON_GET_CLASS (button);
-
-      clutter_ungrab_pointer ();
-
-      button->priv->is_pressed = FALSE;
-
-      if (klass->released)
-        klass->released (button);
+      if (st_widget_get_hover (ST_WIDGET (button)))
+        st_button_press (button);
+      else
+        st_button_release (button, FALSE);
     }
 
-  st_widget_remove_style_pseudo_class ((StWidget*) button, "hover");
-
-  return CLUTTER_ACTOR_CLASS (st_button_parent_class)->leave_event (actor, event);
+  return ret;
 }
 
 static void
@@ -242,7 +272,6 @@ st_button_set_property (GObject      *gobject,
                         GParamSpec   *pspec)
 {
   StButton *button = ST_BUTTON (gobject);
-  StButtonPrivate *priv = ST_BUTTON (gobject)->priv;
 
   switch (prop_id)
     {
@@ -255,9 +284,6 @@ st_button_set_property (GObject      *gobject,
     case PROP_CHECKED:
       st_button_set_checked (button, g_value_get_boolean (value));
       break;
-    case PROP_TRANSITION_DURATION:
-      priv->transition_duration = g_value_get_int (value);
-      break;
 
 
     default:
@@ -285,9 +311,6 @@ st_button_get_property (GObject    *gobject,
     case PROP_CHECKED:
       g_value_set_boolean (value, priv->is_checked);
       break;
-    case PROP_TRANSITION_DURATION:
-      g_value_set_int (value, priv->transition_duration);
-      break;
 
 
     default:
@@ -316,15 +339,14 @@ st_button_class_init (StButtonClass *klass)
 
   g_type_class_add_private (klass, sizeof (StButtonPrivate));
 
-  klass->pressed = st_button_real_pressed;
-  klass->released = st_button_real_released;
-
   gobject_class->set_property = st_button_set_property;
   gobject_class->get_property = st_button_get_property;
   gobject_class->finalize = st_button_finalize;
 
   actor_class->button_press_event = st_button_button_press;
   actor_class->button_release_event = st_button_button_release;
+  actor_class->key_press_event = st_button_key_press;
+  actor_class->key_release_event = st_button_key_release;
   actor_class->enter_event = st_button_enter;
   actor_class->leave_event = st_button_leave;
 
@@ -349,12 +371,6 @@ st_button_class_init (StButtonClass *klass)
                                 FALSE, G_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHECKED, pspec);
 
-  pspec = g_param_spec_int ("transition-duration",
-                            "Transition Duration",
-                            "Duration of the state transition effect",
-                            0, G_MAXINT, 120, G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_TRANSITION_DURATION, pspec);
-
 
   /**
    * StButton::clicked:
@@ -378,10 +394,10 @@ static void
 st_button_init (StButton *button)
 {
   button->priv = ST_BUTTON_GET_PRIVATE (button);
-  button->priv->transition_duration = 120;
   button->priv->spacing = 6;
 
-  clutter_actor_set_reactive ((ClutterActor *) button, TRUE);
+  clutter_actor_set_reactive (CLUTTER_ACTOR (button), TRUE);
+  st_widget_set_track_hover (ST_WIDGET (button), TRUE);
 }
 
 /**
@@ -452,7 +468,7 @@ st_button_set_label (StButton    *button,
   else
     priv->text = g_strdup ("");
 
-  label = st_bin_get_child ((StBin*) button);
+  label = st_bin_get_child (ST_BIN (button));
 
   if (label && CLUTTER_IS_TEXT (label))
     {
@@ -466,7 +482,7 @@ st_button_set_label (StButton    *button,
                             "ellipsize", PANGO_ELLIPSIZE_END,
                             "use-markup", TRUE,
                             NULL);
-      st_bin_set_child ((StBin*) button, label);
+      st_bin_set_child (ST_BIN (button), label);
     }
 
   /* Fake a style change so that we reset the style properties on the label */
@@ -545,9 +561,9 @@ st_button_set_checked (StButton *button,
       button->priv->is_checked = checked;
 
       if (checked)
-        st_widget_add_style_pseudo_class ((StWidget*) button, "checked");
+        st_widget_add_style_pseudo_class (ST_WIDGET (button), "checked");
       else
-        st_widget_remove_style_pseudo_class ((StWidget*) button, "checked");
+        st_widget_remove_style_pseudo_class (ST_WIDGET (button), "checked");
     }
 
   g_object_notify (G_OBJECT (button), "checked");
