@@ -202,13 +202,13 @@ st_entry_style_changed (StWidget *self)
   st_theme_node_get_foreground_color (theme_node, &color);
   clutter_text_set_color (CLUTTER_TEXT (priv->entry), &color);
 
-  if (st_theme_node_get_length (theme_node, "caret-size", FALSE, &size))
+  if (st_theme_node_lookup_length (theme_node, "caret-size", FALSE, &size))
     clutter_text_set_cursor_size (CLUTTER_TEXT (priv->entry), (int)(.5 + size));
 
-  if (st_theme_node_get_color (theme_node, "caret-color", FALSE, &color))
+  if (st_theme_node_lookup_color (theme_node, "caret-color", FALSE, &color))
     clutter_text_set_cursor_color (CLUTTER_TEXT (priv->entry), &color);
 
-  if (st_theme_node_get_color (theme_node, "selection-background-color", FALSE, &color))
+  if (st_theme_node_lookup_color (theme_node, "selection-background-color", FALSE, &color))
     clutter_text_set_selection_color (CLUTTER_TEXT (priv->entry), &color);
 
   font = st_theme_node_get_font (theme_node);
@@ -219,6 +219,29 @@ st_entry_style_changed (StWidget *self)
   ST_WIDGET_CLASS (st_entry_parent_class)->style_changed (self);
 }
 
+static gboolean
+st_entry_navigate_focus (StWidget         *widget,
+                         ClutterActor     *from,
+                         GtkDirectionType  direction)
+{
+  StEntryPrivate *priv = ST_ENTRY_PRIV (widget);
+
+  /* This is basically the same as st_widget_real_navigate_focus(),
+   * except that widget is behaving as a proxy for priv->entry (which
+   * isn't an StWidget and so has no can-focus flag of its own).
+   */
+
+  if (from == priv->entry)
+    return FALSE;
+  else if (st_widget_get_can_focus (widget))
+    {
+      clutter_actor_grab_key_focus (priv->entry);
+      return TRUE;
+    }
+  else
+    return FALSE;
+}
+
 static void
 st_entry_get_preferred_width (ClutterActor *actor,
                               gfloat        for_height,
@@ -584,7 +607,7 @@ st_entry_enter_event (ClutterActor         *actor,
 {
   StEntryPrivate *priv = ST_ENTRY_PRIV (actor);
 
-  if (_st_actor_contains (actor, event->source)
+  if (clutter_actor_contains (actor, event->source)
       && priv->hint && priv->hint_visible)
     {
       st_widget_set_hover (ST_WIDGET (actor), TRUE);
@@ -597,7 +620,7 @@ static gboolean
 st_entry_leave_event (ClutterActor         *actor,
                       ClutterCrossingEvent *event)
 {
-  if (!_st_actor_contains (actor, event->related))
+  if (!clutter_actor_contains (actor, event->related))
     st_widget_set_hover (ST_WIDGET (actor), FALSE);
 
   return CLUTTER_ACTOR_CLASS (st_entry_parent_class)->leave_event (actor, event);
@@ -632,6 +655,7 @@ st_entry_class_init (StEntryClass *klass)
   actor_class->leave_event = st_entry_leave_event;
 
   widget_class->style_changed = st_entry_style_changed;
+  widget_class->navigate_focus = st_entry_navigate_focus;
 
   pspec = g_param_spec_object ("clutter-text",
 			       "Clutter Text",
