@@ -22,6 +22,10 @@ struct _StThemeNode {
   StGradientType background_gradient_type;
   ClutterColor background_gradient_end;
 
+  int background_position_x;
+  int background_position_y;
+  gboolean background_position_set : 1;
+
   ClutterColor foreground_color;
   ClutterColor border_color[4];
   ClutterColor outline_color;
@@ -43,6 +47,7 @@ struct _StThemeNode {
   char *background_image;
   StBorderImage *border_image;
   StShadow *shadow;
+  StShadow *text_shadow;
 
   GType element_type;
   char *element_id;
@@ -62,6 +67,7 @@ struct _StThemeNode {
   guint foreground_computed : 1;
   guint border_image_computed : 1;
   guint shadow_computed : 1;
+  guint text_shadow_computed : 1;
   guint link_type : 2;
 
   /* Graphics state */
@@ -71,8 +77,10 @@ struct _StThemeNode {
   CoglHandle background_shadow_material;
   CoglHandle border_shadow_material;
   CoglHandle background_texture;
+  CoglHandle background_material;
   CoglHandle border_texture;
-  CoglHandle corner_texture;
+  CoglHandle border_material;
+  CoglHandle corner_material[4];
 };
 
 struct _StThemeNodeClass {
